@@ -91,6 +91,17 @@ void AdvSceneSwitcher::CooldownDurationChanged(const Duration &dur)
 	switcher->cooldown = dur;
 }
 
+void AdvSceneSwitcher::on_enableCooldown_stateChanged(int state)
+{
+	if (loading) {
+		return;
+	}
+
+	std::lock_guard<std::mutex> lock(switcher->m);
+	switcher->enableCooldown = state;
+	ui->cooldownTime->setEnabled(state);
+}
+
 void AdvSceneSwitcher::on_startupBehavior_currentIndexChanged(int index)
 {
 	if (loading) {
@@ -139,6 +150,8 @@ void AdvSceneSwitcher::on_checkInterval_valueChanged(int value)
 
 	std::lock_guard<std::mutex> lock(switcher->m);
 	switcher->interval = value;
+
+	SetCheckIntervalTooLowVisibility();
 }
 
 void AdvSceneSwitcher::closeEvent(QCloseEvent *)
@@ -205,7 +218,6 @@ static bool isLegacyTab(const QString &name)
 {
 	return name == obs_module_text(
 			       "AdvSceneSwitcher.sceneGroupTab.title") ||
-	       name == obs_module_text("AdvSceneSwitcher.networkTab.title") ||
 	       name == obs_module_text(
 			       "AdvSceneSwitcher.transitionTab.title") ||
 	       name == obs_module_text(
@@ -223,9 +235,7 @@ static bool isLegacyTab(const QString &name)
 			       "AdvSceneSwitcher.sceneSequenceTab.title") ||
 	       name == obs_module_text("AdvSceneSwitcher.audioTab.title") ||
 	       name == obs_module_text("AdvSceneSwitcher.videoTab.title") ||
-	       name == obs_module_text("AdvSceneSwitcher.pauseTab.title") ||
-	       name == obs_module_text(
-			       "AdvSceneSwitcher.sceneTriggerTab.title");
+	       name == obs_module_text("AdvSceneSwitcher.pauseTab.title");
 }
 
 void AdvSceneSwitcher::on_hideLegacyTabs_stateChanged(int state)
@@ -237,6 +247,10 @@ void AdvSceneSwitcher::on_hideLegacyTabs_stateChanged(int state)
 			ui->tabWidget->setTabVisible(idx, !state);
 		}
 	}
+
+	// Changing priority of legacy tabs will very likely not be necessary if
+	// the legacy tabs are hidden
+	ui->priorityBox->setVisible(!switcher->hideLegacyTabs);
 }
 
 void AdvSceneSwitcher::SetDeprecationWarnings()
@@ -445,8 +459,6 @@ void SwitcherData::LoadSettings(obs_data_t *obj)
 	loadTimeSwitches(obj);
 	loadAudioSwitches(obj);
 	loadVideoSwitches(obj);
-	loadNetworkSettings(obj);
-	loadSceneTriggers(obj);
 	LoadGeneralSettings(obj);
 	LoadHotkeys(obj);
 	LoadUISettings(obj);
@@ -481,8 +493,6 @@ void SwitcherData::SaveSettings(obs_data_t *obj)
 	saveTimeSwitches(obj);
 	saveAudioSwitches(obj);
 	saveVideoSwitches(obj);
-	saveNetworkSwitches(obj);
-	saveSceneTriggers(obj);
 	SaveGeneralSettings(obj);
 	SaveHotkeys(obj);
 	SaveUISettings(obj);
@@ -505,6 +515,7 @@ void SwitcherData::SaveGeneralSettings(obs_data_t *obj)
 	noMatchDelay.Save(obj, "noMatchDelay");
 
 	cooldown.Save(obj, "cooldown");
+	obs_data_set_bool(obj, "enableCooldown", enableCooldown);
 
 	obs_data_set_bool(obj, "active", sceneColletionStop ? true : !stop);
 	sceneColletionStop = false;
@@ -515,6 +526,7 @@ void SwitcherData::SaveGeneralSettings(obs_data_t *obj)
 			 static_cast<int>(autoStartEvent));
 
 	obs_data_set_int(obj, "logLevel", static_cast<int>(logLevel));
+	obs_data_set_int(obj, "logLevelVersion", 1);
 	obs_data_set_bool(obj, "showSystemTrayNotifications",
 			  showSystemTrayNotifications);
 	obs_data_set_bool(obj, "disableHints", disableHints);
@@ -550,6 +562,11 @@ void SwitcherData::LoadGeneralSettings(obs_data_t *obj)
 	noMatchDelay.Load(obj, "noMatchDelay");
 
 	cooldown.Load(obj, "cooldown");
+	if (!obs_data_has_user_value(obj, "enableCooldown")) {
+		enableCooldown = cooldown.Seconds() != 0;
+	} else {
+		enableCooldown = obs_data_get_bool(obj, "enableCooldown");
+	}
 
 	obs_data_set_default_bool(obj, "active", true);
 	stop = !obs_data_get_bool(obj, "active");
@@ -566,6 +583,25 @@ void SwitcherData::LoadGeneralSettings(obs_data_t *obj)
 		static_cast<AutoStart>(obs_data_get_int(obj, "autoStartEvent"));
 
 	logLevel = static_cast<LogLevel>(obs_data_get_int(obj, "logLevel"));
+	if (obs_data_get_int(obj, "logLevelVersion") < 1) {
+		enum OldLogLevel { DEFAULT, LOG_ACTION, VERBOSE };
+		OldLogLevel oldLogLevel = static_cast<OldLogLevel>(
+			obs_data_get_int(obj, "logLevel"));
+		switch (oldLogLevel) {
+		case DEFAULT:
+			logLevel = LogLevel::DEFAULT;
+			break;
+		case LOG_ACTION:
+			logLevel = LogLevel::LOG_ACTION;
+			break;
+		case VERBOSE:
+			logLevel = LogLevel::VERBOSE;
+			break;
+		default:
+			break;
+		}
+	}
+
 	showSystemTrayNotifications =
 		obs_data_get_bool(obj, "showSystemTrayNotifications");
 	disableHints = obs_data_get_bool(obj, "disableHints");
@@ -653,7 +689,7 @@ void SwitcherData::CheckNoMatchSwitch(bool &match, OBSWeakSource &scene,
 
 void SwitcherData::checkSwitchCooldown(bool &match)
 {
-	if (!match) {
+	if (!match || !enableCooldown) {
 		return;
 	}
 
@@ -793,6 +829,28 @@ static void setupGeneralTabInactiveWarning(QTabWidget *tabs)
 	inactiveTimer->start();
 }
 
+void advss::AdvSceneSwitcher::SetCheckIntervalTooLowVisibility() const
+{
+	auto macro = GetMacroWithInvalidConditionInterval();
+	if (!macro) {
+		ui->checkIntervalTooLowWarning->hide();
+		return;
+	}
+
+	const QString labelTextFormat(obs_module_text(
+		"AdvSceneSwitcher.generalTab.status.checkIntervalTooLow"));
+	const QString labelTooltipFormat(obs_module_text(
+		"AdvSceneSwitcher.generalTab.status.checkIntervalTooLow.tooltip"));
+	const QString name = QString::fromStdString(macro->Name());
+	const QString duration = QString::fromStdString(
+		macro->GetCustomConditionCheckInterval().ToString());
+
+	ui->checkIntervalTooLowWarning->setText(labelTextFormat.arg(name));
+	ui->checkIntervalTooLowWarning->setToolTip(
+		labelTooltipFormat.arg(name).arg(duration).arg(name));
+	ui->checkIntervalTooLowWarning->show();
+}
+
 void AdvSceneSwitcher::SetupGeneralTab()
 {
 	PopulateSceneSelection(ui->noMatchSwitchScene, false);
@@ -814,14 +872,17 @@ void AdvSceneSwitcher::SetupGeneralTab()
 	DurationSelection *noMatchDelay = new DurationSelection();
 	noMatchDelay->SetDuration(switcher->noMatchDelay);
 	noMatchDelay->setToolTip(obs_module_text(
-		"AdvSceneSwitcher.generalTab.generalBehavior.onNoMetDelayTooltip"));
+		"AdvSceneSwitcher.generalTab.generalBehavior.onNoMatchDelay.tooltip"));
 	ui->noMatchLayout->addWidget(noMatchDelay);
 	QWidget::connect(noMatchDelay,
 			 SIGNAL(DurationChanged(const Duration &)), this,
 			 SLOT(NoMatchDelayDurationChanged(const Duration &)));
 
 	ui->checkInterval->setValue(switcher->interval);
+	SetCheckIntervalTooLowVisibility();
 
+	ui->enableCooldown->setChecked(switcher->enableCooldown);
+	ui->cooldownTime->setEnabled(switcher->enableCooldown);
 	ui->cooldownTime->SetDuration(switcher->cooldown);
 	ui->cooldownTime->setToolTip(obs_module_text(
 		"AdvSceneSwitcher.generalTab.generalBehavior.cooldownHint"));
